/* -*- tab-width: 4 -*- */

/* provide, require, and related primitives.
 */

#include "elkkernel.h"

static ELKObject Features;

void Init_Features (void) {
	Features = Null;
	Global_GC_Link (Features);
#ifdef CAN_DUMP
	P_Provide (Intern (x_("elk:dump")));
#endif
#ifdef CAN_LOAD_OBJ
	P_Provide (Intern (x_("elk:load-object")));
#endif
}

ELKObject P_Features (void) {
	return Features;
}

ELKObject P_Featurep (ELKObject sym) {
	ELKObject member;

	Check_Type (sym, T_Symbol);
	member = P_Memq (sym, Features);
	return Truep (member) ? Trueob : Falseob;
}

ELKObject P_Provide (ELKObject sym) {
	ELKObject member;

	Check_Type (sym, T_Symbol);
	member = P_Memq (sym, Features);
	if (!Truep (member))
		Features = Cons (sym, Features);
	return Void;
}

static ELKObject Feature_Filename (ELKObject str) {
	struct S_String *sp = STRING(str);
	int len = sp->size;
	CHAR *p;
	ELKObject s;
	GC_Node;

	for (p = sp->data+len-1; p >= sp->data && *p != '.'; p--)
		;
	if (p >= sp->data)
	   return str;
	GC_Link (str);
	s = Make_String ((CHAR *)0, len+4);
	bcopy (STRING(str)->data, STRING(s)->data, len);
	bcopy (x_(".scm"), STRING(s)->data+len, 4);
	GC_Unlink;
	return s;
}

ELKObject P_Require (int argc, ELKObject *argv) {
	ELKObject sym, a[1], isfeature;
	GC_Node;

	sym = argv[0];
	GC_Link (sym);
	isfeature = P_Featurep (sym);
	if (!Truep (isfeature)) {
		if (argc == 3)
			Check_Type (argv[2], T_Environment);
		a[0] = argc == 1 ? Feature_Filename (SYMBOL(sym)->name) : argv[1];
		if (Var_Is_True (V_Autoload_Notifyp))
			Format (Standard_Output_Port, x_("[Autoloading ~a]~%"), 18, 1, a);
		(void)General_Load (a[0], argc == 3 ? argv[2] : The_Environment);
		isfeature = P_Featurep (sym);
		if (!Truep (isfeature))
			Primitive_Error (x_("feature ~s was not provided"), sym);
	}
	GC_Unlink;
	return Void;
}
