/* -*- tab-width: 4 -*- */

/* Built-in and user-defined Scheme types.
 */

#include "elkkernel.h"

#define TYPE_GROW	10

TYPEDESCR *Types;
int Num_Types, Max_Type;

CHAR *builtin_types[] = {
	x_("0integer"), x_("1integer") /* bignum */, x_("1real"), x_("0null"), x_("0boolean"),
	x_("0unbound"), x_("0special"), x_("0character"), x_("1symbol"), x_("1pair"),
	x_("1environment"), x_("1string"), x_("1vector"), x_("1primitive"), x_("1compound"),
	x_("1control-point"), x_("1promise"), x_("1port"), x_("0end-of-file"), x_("1autoload"),
	x_("1macro"), x_("1!!broken-heart!!"),
#ifdef GENERATIONAL_GC
	x_("0align_8byte"), x_("0freespace"),
#endif
	0
};

void Wrong_Type (ELKObject x, register int t) {
	Wrong_Type_Combination (x, Types[t].name);
}

void Wrong_Type_Combination (ELKObject x, register const CHAR *name) {
	register int t = TYPE(x);
	CHAR buf[100];

	if (t < 0 || t >= Num_Types)
		Panic (x_("bad type1"));
	esnprintf (buf, 100, x_("wrong argument type %s (expected %s)"),
		Types[t].name, name);
	Primitive_Error (buf);
}

ELKObject P_Type (ELKObject x) {
	register int t = TYPE(x);

	if (t < 0 || t >= Num_Types)
		Panic (x_("bad type2"));
	return Intern (Types[t].name);
}

int Define_Type (int t, const CHAR *name, int (*size)(ELKObject), int const_size,
	int (*eqv)(ELKObject, ELKObject), int (*equal)(ELKObject, ELKObject),
		int (*print)(ELKObject, ELKObject, int, int, int), int (*visit)(ELKObject*, void (*)(ELKObject*)))
{
	register TYPEDESCR *p;

	Set_Error_Tag (x_("define-type"));
	if (t != 0)
		Fatal_Error(x_("first arg of Define_Type() must be 0"));
	if (Num_Types == Max_Type) {
		Max_Type += TYPE_GROW;
		Types = (TYPEDESCR *)Safe_Realloc((CHAR *)Types,
			Max_Type * sizeof(TYPEDESCR));
	}
	Disable_Interrupts;
	p = &Types[Num_Types++];
	p->haspointer = 1;
	p->name = name;
	p->size = size;
	p->const_size = const_size;
	p->eqv = eqv;
	p->equal = equal;
	p->print = print;
	p->visit = visit;
	Enable_Interrupts;
	return Num_Types-1;
}

void Init_Type(void) {
	int i, bytes;
	CHAR *p;

	Num_Types = (sizeof(builtin_types) - 1) / sizeof(CHAR *);
	Max_Type = Num_Types + TYPE_GROW;
	bytes = Max_Type * sizeof(TYPEDESCR);
	Types = (TYPEDESCR *)Safe_Malloc(bytes);
	bzero((CHAR *)Types, bytes);
	for (i = 0; (p = builtin_types[i]) != 0; i++) {
		Types[i].haspointer = *p != '0';
		Types[i].name = ++p;
	}
}
