/* -*- tab-width: 4 -*- */

#include "elkkernel.h"

ELKObject General_Make_Vector (int len, ELKObject fill, int konst);

ELKObject General_Make_Vector (int len, ELKObject fill, int konst) {
	ELKObject vec;
	register ELKObject *op;
	GC_Node;

	GC_Link (fill);
	vec = Alloc_Object ((len-1) * sizeof (ELKObject) + sizeof (struct S_Vector),
		T_Vector, konst);
	VECTOR(vec)->tag = Null;
	VECTOR(vec)->size = len;
	for (op = VECTOR(vec)->data; len--; op++)
		*op = fill;
	GC_Unlink;
	return vec;
}

ELKObject Make_Vector (int len, ELKObject fill) {
	return General_Make_Vector (len, fill, 0);
}

ELKObject Make_Const_Vector (int len, ELKObject fill) {
	return General_Make_Vector (len, fill, 1);
}

ELKObject P_Make_Vector (int argc, ELKObject *argv) {
	register int len;

	if ((len = Get_Exact_Integer (argv[0])) < 0)
		Range_Error (argv[0]);
	return Make_Vector (len, argc == 1 ? Null : argv[1]);
}

ELKObject P_Vector (int argc, ELKObject *argv) {
	ELKObject vec;
	register int i;

	vec = Make_Vector (argc, Null);
	for (i = 0; i < argc; i++)
		VECTOR(vec)->data[i] = *argv++;
	return vec;
}

ELKObject P_Vectorp (ELKObject x) {
	return TYPE(x) == T_Vector ? Trueob : Falseob;
}

ELKObject P_Vector_Length (ELKObject x) {
	Check_Type (x, T_Vector);
	return Make_Integer (VECTOR(x)->size);
}

ELKObject P_Vector_Ref (ELKObject vec, ELKObject n) {
	Check_Type (vec, T_Vector);
	return VECTOR(vec)->data[Get_Index (n, vec)];
}

ELKObject P_Vector_Set (ELKObject vec, ELKObject n, ELKObject newv) {
	ELKObject old;
	register int i;

	Check_Type (vec, T_Vector);
	Check_Mutable (vec);
	old = VECTOR(vec)->data[i = Get_Index (n, vec)];
	VECTOR(vec)->data[i] = newv;
	return old;
}

/* We cannot simply call P_List with vec->size and vec->data here,
 * because the latter can change during GC.
 */
ELKObject P_Vector_To_List (ELKObject vec) {
	register int i;
	ELKObject list, tail, cell;
	GC_Node3;

	Check_Type (vec, T_Vector);
	list = tail = Null;
	GC_Link3 (vec, list, tail);
	for (i = 0; i < VECTOR(vec)->size; i++, tail = cell) {
		cell = Cons (VECTOR(vec)->data[i], Null);
		if (Nullp (list))
			list = cell;
		else
			(void)P_Set_Cdr (tail, cell);
	}
	GC_Unlink;
	return list;
}

ELKObject List_To_Vector (ELKObject list, int konst) {
	ELKObject vec, len;
	register int i;
	GC_Node;

	GC_Link (list);
	len = P_Length (list);
	if (konst)
		vec = Make_Const_Vector (FIXNUM(len), Null);
	else
		vec = Make_Vector (FIXNUM(len), Null);
	for (i = 0; i < FIXNUM(len); i++, list = Cdr (list))
		VECTOR(vec)->data[i] = Car (list);
	GC_Unlink;
	return vec;
}

ELKObject P_List_To_Vector (ELKObject list) {
	return List_To_Vector (list, 0);
}

ELKObject P_Vector_Fill (ELKObject vec, ELKObject fill) {
	register int i;

	Check_Type (vec, T_Vector);
	Check_Mutable (vec);
	for (i = 0; i < VECTOR(vec)->size; i++)
		VECTOR(vec)->data[i] = fill;
	return vec;
}

ELKObject P_Vector_Copy (ELKObject vec) {
	ELKObject newo;
	GC_Node;

	Check_Type (vec, T_Vector);
	GC_Link (vec);
	newo = Make_Vector (VECTOR(vec)->size, Null);
	bcopy ((CHAR *)POINTER(vec), (CHAR *)POINTER(newo),
		(VECTOR(vec)->size-1) * sizeof (ELKObject) + sizeof (struct S_Vector));
	GC_Unlink;
	return newo;
}
